<?php
/** 
 * @package   	VikWP - Libraries
 * @subpackage 	adapter.database
 * @author    	E4J s.r.l.
 * @copyright 	Copyright (C) 2019 E4J s.r.l. All Rights Reserved.
 * @license  	http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @link 		https://vikwp.com
 */

// No direct access
defined('ABSPATH') or die('No script kiddies please!');

/**
 * Generic table class for database objects.
 *
 * @since 10.1.19
 */
class JTable
{
	/**
	 * Include paths for searching for Table classes.
	 *
	 * @var array
	 */
	private static $_includePaths = array();

	/**
	 * Name of the database table to model.
	 *
	 * @var string
	 */
	protected $_table = '';

	/**
	 * Name of the primary key fields in the table.
	 *
	 * @var array
	 */
	protected $_tableKeys = array();

	/**
	 * Indicates that the primary keys autoincrement.
	 *
	 * @var boolean
	 */
	protected $_autoincrement = true;

	/**
	 * Static method to get an instance of a Table class if it
	 * can be found in the table include paths.
	 *
	 * @param   string  $type    The type (name) of the Table class to get an instance of.
	 * @param   string  $prefix  An optional prefix for the table class name.
	 * @param   array   $config  An optional array of configuration values for the Table object.
	 *
	 * @return  mixed   A Table object if found, false otherwise.
	 *
	 * @see 	JTable::addIncludePath() to add include paths for searching for Table classes.
	 */
	public static function getInstance($type, $prefix = 'JTable', $config = array())
	{
		// sanitize and prepare the table class name
		$type       = preg_replace('/[^A-Z0-9_\.-]/i', '', $type);
		$tableClass = $prefix . ucfirst($type);

		// only try to load the class if it doesn't already exist
		if (!class_exists($tableClass))
		{
			// search for the class file in the JTable include paths.
			$paths = static::addIncludePath();
			$pathIndex = 0;

			// iterate until the class is loaded
			while (!class_exists($tableClass) && $pathIndex < count($paths))
			{
				if ($tryThis = JPath::find($paths[$pathIndex++], strtolower($type) . '.php'))
				{
					// import the class file
					include_once $tryThis;
				}
			}

			if (!class_exists($tableClass))
			{
				throw new Exception(sprintf('Table [%s] not found', $tableClass), 404);
			}
		}

		// instantiate a new table class and return it
		return new $tableClass($type);
	}

	/**
	 * Adds a filesystem path where Table should search for table class files.
	 *
	 * @param   mixed  $path  A filesystem path or array of filesystem paths to add.
	 *
	 * @return  array  An array of filesystem paths to find Table classes in.
	 */
	public static function addIncludePath($path = null)
	{
		// if the internal paths have not been initialised, do so with the base table path
		if (empty(self::$_includePaths))
		{
			self::$_includePaths[] = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'tables';
		}

		if ($path)
		{
			// convert the passed path(s) to add to an array.
			$path = (array) $path;

			// add each individual new path
			foreach ($path as $dir)
			{
				// sanitize path
				$dir = trim($dir);

				// add to the front of the list so that custom paths are searched first
				if (!in_array($dir, self::$_includePaths))
				{
					array_unshift(self::$_includePaths, $dir);
				}
			}
		}

		return self::$_includePaths;
	}

	/**
	 * Object constructor to set table and key fields.
	 * In most cases this will be overridden by child classes to explicitly
	 * set the table and key fields for a particular database table.
	 *
	 * @param   string 	$table  Name of the table to model.
	 * @param   mixed 	$key    Name of the primary key field in the table 
	 * 							or array of field names that compose the primary key.
	 */
	public function __construct($table, $key = 'id')
	{
		// set internal variables
		$this->_table = $table;

		// set the key to be an array.
		if (is_string($key))
		{
			$key = array($key);
		}
		else if (is_object($key))
		{
			$key = (array) $key;
		}

		$this->_tableKeys = $key;

		if (count($key) == 1)
		{
			$this->_autoincrement = true;
		}
		else
		{
			$this->_autoincrement = false;
		}

		// initialise the table properties
		$fields = $this->getFields();

		if ($fields)
		{
			foreach ($fields as $name => $v)
			{
				if (is_int($name))
				{
					// use the value in case we have a linear array
					$name = $v;
				}

				// add the field if it is not already present
				if (!property_exists($this, $name))
				{
					$this->{$name} = null;
				}
			}
		}
	}

	/**
	 * Gets the columns from database table.
	 *
	 * @param   boolean  $reload  Flag to reload cache.
	 *
	 * @return  mixed  	 An array of the field names, or false if an error occurs.
	 *
	 * @throws  Exception
	 */
	public function getFields($reload = false)
	{
		static $cache = null;

		if ($cache === null || $reload)
		{
			$dbo = JFactory::getDbo();

			// lookup the fields for this table only once
			$fields = $dbo->getTableColumns($this->_table, false);

			if (empty($fields))
			{
				throw new Exception(sprintf('No columns found for [%s] table', $this->_table));
			}

			$cache = $fields;
		}

		return $cache;
	}
}
